#import "Program.h"
#import "ProgramTable.h"

@implementation ProgramTable

-(ProgramTable*) initWithChannelName: (NSString*) name {
	[ self initWithChannelNameAndURL : name : @"" ];
	return self;
}
-(ProgramTable*) initWithChannelNameAndURL: (NSString*) name : (NSString*) url {
	[ super init ];
	channelName = name;
	[ channelName retain ];
	programs = [ [ NSMutableArray alloc ] init ];
	[ programs retain ];
	address = url;
	[ address retain ];
	arranged = YES;
	return self;
}

-(void) dealloc {
	[ super dealloc ];
	[ channelName release ];
	[ programs release ];
}

-(void)setAddress : (NSString*) url {
	[ address release ];
	address = url;
	[ address retain ];
}

-(NSString*)getAddress {
	return address;
}

-(int)  getProgramSize {
	return [ programs count ];
}

-(Program*) getProgram : (int) num {
	if ( num < 0 || num >= [ programs count ] ) return nil;
	//NSLog( @"Request : %d", num );
	return [ programs objectAtIndex: num ];
}

-(NSArray*) getPrograms {
	return programs;
}

-(NSArray*) getProgramWithTimeRange : (int)startMin minutes:(int)minutes {
	int i;
	NSMutableArray* progs = [ [ NSMutableArray alloc ] init ];
	[ self arrange ];
	for ( i = 0; i < [ programs count ]; i++ ) {
		Program* p = [ programs objectAtIndex: i ];
		int t0 = [ p getStartTime ];
		int t1 = [ p getEndTime ];
//		NSLog( @"%d - %d ( %d )\t%@", t0, t1, minutes, [ p getTitle ] );
		if ( t1 <= startMin || t0 > ( startMin + minutes ) ) continue;
		[ progs addObject: p ];
	}
	return progs;
}

-(void)arrange {
	if ( arranged ) return;
	[ programs sortUsingFunction : _Program__comparator context : nil ];
	arranged = YES;
}

-(NSString*)getChannelName {
	return channelName;
}

-(void) appendProgram : (Program*) program {
	[ programs addObject: program ];
	arranged = NO;
}

-(NSString*) description {
	int i;
	NSMutableString*  output = [ NSMutableString stringWithFormat :@"\n%@ : %d progs\n", channelName, [ self getProgramSize ] ];
	[ self arrange ];
	for ( i = 0; i < [ self getProgramSize ]; i++ ) {
		//    NSLog( @"%d ==> %@", i, [ self getProgram: i ] );
		[ output appendString: [ NSString stringWithFormat : @"%@\n", [ self getProgram: i ] ] ];//[ programs objectAtIndex: i ] ] ];
  }
	return output;
}

/*
 $channel$
 $program<time:title:description:comment:genre>$
 
 <!--$channel$-->
 $program-start$
 <div class="prog" class="$genre$">
 <span class="time" style="height:$length$">$starttime$</span>
 $title$
 $description$
 </div>
 $program-end$
 */
-(NSString*)toHTML : (NSString*)format start : (int)startTime end : (int)endTime {
	NSMutableString* output = [ NSMutableString stringWithString: format ];
	NSRange range_channel;
	NSRange range_search;
	
	//NSLog( @"OUTPUT %s:%d\n%@", __func__, __LINE__, output );
	
	range_search.location = 0;
	//output = [ [ NSMutableString alloc ] init ];
	range_channel = [ output rangeOfString: @"$channel$" ];
	/* replace channel name */
	while ( range_channel.location != NSNotFound ) {
		//NSLog( @"RANGE %d - %d", range_channel.location, range_channel.length );
		[ output replaceCharactersInRange: range_channel withString: self->channelName ];
		
		//NSLog( @"OUTPUT %s:%d\n%@", __func__, __LINE__, output );
		
		range_search.location = range_channel.location + [ channelName length ];
		range_search.length   = [ output length ] - range_search.location;
		range_channel = [ output rangeOfString: @"$channel$" options:0 range:range_search ];
	}
	
	//NSLog( @"OUTPUT %s:%d\n%@", __func__, __LINE__, output );
	
	/* replace program */
	{
		NSRange prog_start;
		NSRange prog_end;
		NSRange range_prog;
		NSString* prog_format;
		int i;
		NSArray* progs = [ self getProgramWithTimeRange : startTime
												 minutes:endTime - startTime ];
		NSMutableString* pstr = [ [ NSMutableString alloc ] init ];
		prog_start = [ output rangeOfString: @"$program-start$" ];
		prog_end = [ output rangeOfString: @"$program-end$" ];
		
		//NSLog( @"OUTPUT :: %@\nR1, %d, %d \t R2 %d, %d", output, prog_start.location, prog_start.length, prog_end.location, prog_end.length );
		
		range_prog.location = prog_start.location + prog_start.length;
		range_prog.length = prog_end.location - range_prog.location;
		if ( prog_start.location != NSNotFound && prog_end.location != NSNotFound ) {
			prog_format = [ output substringWithRange: range_prog ];
			//NSLog( @"\n\n\n%@\nCOUNT %d\n\n", prog_format, [ progs count ] );
			for ( i = 0; i < [ progs count ]; i++ ) {
				NSString* prog_unit = [ [ progs objectAtIndex : i ] toHTML: prog_format ];
				[ pstr appendString: prog_unit ];
			}
			range_prog.location = prog_start.location;
			range_prog.length = prog_end.location + prog_end.length - prog_start.location;
			[ output replaceCharactersInRange :range_prog withString:pstr ];
		}
	}  
	return output;
}

@end


