#!/usr/bin/python
# -*- coding:utf-8 -*-

""" Convert radio programs retrieved from Yahoo TV to CSV file

Usage : python r2c htmlfile 
"""
import os, string, re, sys
from dom import *

class RadioProgram:
    def __init__( self, starttime, length, title, description ):
        self.starttime   = starttime
        self.length      = length
        self.title       = title
        self.description = description
        self.genre = 0
        pass

    def get_start_time( self ):
        return self.starttime

    def get_end_time( self ):
        return self.starttime + self.length

    def get_legnth( self ):
        return self.length

    def is_spacer( self ): return ( self.title == None )
    def __newInstance( self, rowspan, text, flag_midnight = False ):
        items = re.split( ',', text )
        
        if len( items ) < 2:
            mbn = text.strip()
            if len( mbn ) == 0:
                return None
            mbnums = [ '１', '２', '３', '４', '５', '６', '７', '８', '９', '０' ]
            for i in range( 0, 10 ):
                mbn = re.sub( mbnums[ i ], "%d" % i, mbn )
                pass
            starttime = string.atoi( mbn ) * 60
            if flag_midnight and starttime < 300: starttime += 1440
            return RadioProgram( starttime, rowspan, None, None )

        time_span = items[ 0 ]
        time_start = items[ 1 ]
        hr, mn = re.split( ':', time_start )
        time_start = string.atoi( hr ) * 60 + string.atoi( mn )

        if flag_midnight and time_start < 720:
            time_start += 1440
            pass
        elif time_start >= 720:
            flag_midnight = True
            pass
        title = items[ 2 ]
        for i in range( 3, len( items ) ):
            title += items[ i ]
            pass
        desc = ""
        m = re.search( '(.*?)(▽|\\s+|　|「|〜)(.*)', title )
        if m:
            title = m.group( 1 )
            if m.group( 2 ) == '　' or m.group( 2 ) == '〜' :
                desc = m.group( 3 )
            else:
                desc  = m.group( 2 ) + m.group( 3 )
                pass
            pass
        prog = RadioProgram( time_start, rowspan, title, desc )

        return prog

    def __newSpacer( self, length ):
        return RadioProgram( 300, length, None, None )
    
    newWithString = classmethod( __newInstance )
    newSpacer     = classmethod( __newSpacer )
    
    def __repr__( self ):
        if self.is_spacer():
            return "%d" % self.length
        output = "%d,%d,%s,%s,,%d" % ( self.starttime, self.length, self.title, self.description, self.genre )
        return output
    pass

def __timestr_to_minutes( timestr ):
    hr, mn = re.split( ':', timestr )
    return string.atoi( hr ) * 60 + string.atoi( mn )

def convert_radio_html_to_csv( filename_input, ostream = sys.stdout ):
    fn = filename_input
    file_radio = open( fn )
    line = file_radio.readline()

    re_channel = re.compile( "<a target='tv' href='(.*?)'>(.*?)</a>" )
    re_spacer  = re.compile( "<td.*?>\\s*&nbsp;\\s*</td>" )
    re_tagremover = re.compile( "<[a-zA-Z/][^>]+>" )
    keyword_midnight = "<small>0</small>"
    flag_midnight = False

    channels = []

    # read title
    re_title = re.compile( "<title>(.*?)</title>" )
    while line != "":
        m = re_title.search( line )
        if m:
            title = m.group( 1 )
            area = title
            m = re.search( "\[(.*?)/(.*?)/[^\]]+\]", title )
            if m:
                area = m.group( 2 )
                wave = m.group( 1 )
                pass
            m = re.search( "(\\d+)月(\\d+)日", title )
            if m:
                date = m.group( 1 ) + "/" + m.group( 2 )
                pass
            break
        line = file_radio.readline()
        pass
    ostream.write( "#%s,%s,%s\n" % ( area, date, wave ) )

    # skip until channels
    while line != "":
        if line.strip() == "<!--- navi-->":break
        line = file_radio.readline()
        pass

    while line != "":
        line = line.strip()
        if line == "</tr>":break
        m = re_channel.search( line )
        if m:
            link = m.group( 1 )
            name = re_tagremover.sub( "", m.group( 2 ) )
            channels.append( {"name":name, "link":link, "length":0, "progs":[] } )
        else:
            m = re_spacer.search( line )
            if m:
                channels.append( {"name":"spacer", "link":None, "length":0, "progs":[] } )
                pass
            pass
        line = file_radio.readline()
        pass


    parser = DOMConverter()
    parser.feed( "<html>" )
    while line != "":
        if line.strip() == "<!--- link-->": break
        if line.find( '<img alt=' ) >= 0:
            m = re.search( "<img alt='([^']*)'.*?src='[^']+/(\\w+).gif'>", line )
            if m:
                alt = m.group( 1 )
                src = m.group( 2 )
                if src == "news":
                    item = '[N]'
                elif src == "weather":
                    item = "[天]"
                elif src == "saihousou":
                    item = "[再]"
                    pass
                else:
                    item = '[' + alt + ']'
                    pass
                line = line[ 0: m.start() ] + item + line[ m.end() : ]
                pass
        parser.feed( line )
        line = file_radio.readline()
        pass
    parser.feed( "</html>" )

    doc = parser.get_document()
    row = doc.get_element_at_path( "/html/tr" )
    count = 0
    columns = len( channels )
    start = 0
    while start < columns:
        if channels[ start ] != None:
            break
        start += 1
        pass

    minutes_min = 0
    size = len( channels )
    max_start = 0
    flag_midnight = False
    while row:
        flag_spacer = False
        datum = row.get_element_at_path( 'td' )
        
        while datum:
            if datum.has_attribute( 'rowspan' ):
                rowspan = string.atoi( datum.get_attribute( 'rowspan' ) )
                item_arrange = 0
                height_min = channels[ 0 ][ "length" ]
                #lenstr = ""
                for i in range( 0, size ):
                    height = channels[ i ][ "length" ]
                    #lenstr += "%d:%d, " % ( i, height )
                    if height_min > height:
                        height_min = height
                        item_arrange = i
                        pass
                    pass
                offset = 0
                if height_min < max_start:
                    offset = max_start - height_min
                    rowspan += max_start - height_min
                    pass
                else:
                    max_start = height_min
                    pass
                if max_start >= 900: flag_midnight = True
                program = RadioProgram.newWithString( rowspan, datum.get_text( ',' ), flag_midnight )
                progs = channels[ item_arrange ][ "progs" ]
                if len( progs ) > 0 and progs[ -1 ] != None  and program:
                    diff = progs[ -1 ].get_end_time() - program.get_start_time()
                    if diff != 0:
                        if diff < 0 and diff >= -2:
                            progs[ -1 ].length -= diff
                            channels[ item_arrange ][ "length" ] -= diff
                            item_arrange = 0
                            height_min = channels[ 0 ][ "length" ]
                            for i in range( 0, size ):
                                height = channels[ i ][ "length" ]
                                if height_min > height:
                                    height_min = height
                                    item_arrange = i
                                    pass
                                pass
                            pass
                        pass
                    pass
                progs.append( program )
                channels[ item_arrange ][ "length" ] += rowspan
                if channels[ item_arrange ][ "name" ] == "spacer":
                    flag_spacer = True
                    pass
                pass
            try:
                datum = datum.next_sibling()
            except:
                break
            pass

        try:
            row = row.next_sibling( 'tr' )
        except:
            break
        pass


    for i in range( 0, size ):
        if channels[ i ][ 'name' ] != 'spacer':
            ostream.write( "*%s,%s\n" % (channels[ i ][ 'name' ], channels[ i ][ 'link' ] ) )
            progs = channels[ i ][ "progs" ]
            for p in progs:
                if p and p.is_spacer() == False:
                    ostream.write( str( p ) )
                    ostream.write( "\n" )
                pass
            ostream.write( "\n" )
            pass
        pass
    return

if __name__ == "__main__":
    convert_radio_html_to_csv( sys.argv[ 1 ], sys.stdout )
    pass
